@@ -42,6 +42,7 @@ static zend_always_inline void bevent_rw_cb(struct bufferevent *bevent, php_even
 	zval  *arg_self;
 	zval **args[2];
 	zval  *retval_ptr = NULL;
+	php_event_base_t *b;
 	PHP_EVENT_TSRM_DECL
 
 	PHP_EVENT_ASSERT(bev);
@@ -49,6 +50,7 @@ static zend_always_inline void bevent_rw_cb(struct bufferevent *bevent, php_even
 	PHP_EVENT_ASSERT(bevent == bev->bevent);
 	PHP_EVENT_ASSERT(pfci && pfcc);
 	PHP_EVENT_ASSERT(bev->self);
+	PHP_EVENT_ASSERT(bev->base);
 
 	arg_data = bev->data;
 
@@ -83,15 +85,22 @@ static zend_always_inline void bevent_rw_cb(struct bufferevent *bevent, php_even
 		pfci->param_count	 = 2;
 		pfci->no_separation  = 1;
 
-        if (zend_call_function(pfci, pfcc TSRMLS_CC) == SUCCESS
-                && retval_ptr) {
-            zval_ptr_dtor(&retval_ptr);
-        } else {
-            php_error_docref(NULL TSRMLS_CC, E_WARNING,
-                    "An error occurred while invoking the callback");
+		if (zend_call_function(pfci, pfcc TSRMLS_CC) == SUCCESS && retval_ptr) {
+			zval_ptr_dtor(&retval_ptr);
+		} else {
+			if (EG(exception)) {
+				PHP_EVENT_ASSERT(bev->base);
+				PHP_EVENT_FETCH_BASE(b, bev->base);
+				event_base_loopbreak(b->base);
+
+				zval_ptr_dtor(&arg_data);
+			} else {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING,
+						"An error occurred while invoking the callback");
+			}
         }
 
-        zval_ptr_dtor(&arg_data);
+		zval_ptr_dtor(&arg_data);
 
 #ifdef HAVE_EVENT_PTHREADS_LIB
 		if (bevent) {
@@ -132,13 +141,14 @@ static void bevent_event_cb(struct bufferevent *bevent, short events, void *ptr)
 	zval  *arg_self;
 	zval **args[3];
 	zval  *retval_ptr = NULL;
+	php_event_base_t *b;
 	PHP_EVENT_TSRM_DECL
 
 	PHP_EVENT_ASSERT(pfci && pfcc);
 	PHP_EVENT_ASSERT(bevent);
 	PHP_EVENT_ASSERT(bev->bevent == bevent);
 	PHP_EVENT_ASSERT(bev->self);
-
+	PHP_EVENT_ASSERT(bev->base);
 
 	arg_data = bev->data;
 
@@ -178,16 +188,24 @@ static void bevent_event_cb(struct bufferevent *bevent, short events, void *ptr)
 		pfci->param_count	 = 3;
 		pfci->no_separation  = 1;
 
-        if (zend_call_function(pfci, pfcc TSRMLS_CC) == SUCCESS
-                && retval_ptr) {
-            zval_ptr_dtor(&retval_ptr);
-        } else {
-            php_error_docref(NULL TSRMLS_CC, E_WARNING,
-                    "An error occurred while invoking the callback");
-        }
+		if (zend_call_function(pfci, pfcc TSRMLS_CC) == SUCCESS && retval_ptr) {
+			zval_ptr_dtor(&retval_ptr);
+		} else {
+			if (EG(exception)) {
+				PHP_EVENT_ASSERT(bev->base);
+				PHP_EVENT_FETCH_BASE(b, bev->base);
+				event_base_loopbreak(b->base);
 
-        zval_ptr_dtor(&arg_events);
-        zval_ptr_dtor(&arg_data);
+				zval_ptr_dtor(&arg_events);
+				zval_ptr_dtor(&arg_data);
+			} else {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING,
+						"An error occurred while invoking the callback");
+			}
+		}
+
+		zval_ptr_dtor(&arg_events);
+		zval_ptr_dtor(&arg_data);
 
 		PHP_EVENT_ASSERT(bevent);
 #ifdef HAVE_EVENT_PTHREADS_LIB
@@ -297,6 +315,9 @@ PHP_METHOD(EventBufferEvent, __construct)
 	bev->self = zself;
 	Z_ADDREF_P(zself);
 
+	bev->base = zbase;
+	Z_ADDREF_P(zbase);
+
 	bev->input = bev->output = NULL;
 
 	if (ZEND_FCI_INITIALIZED(fci_read)) {
@@ -364,10 +385,36 @@ PHP_METHOD(EventBufferEvent, free)
 			zval_ptr_dtor(&bev->self);
 			bev->self = NULL;
 		}
+		if (bev->base) {
+			zval_ptr_dtor(&bev->base);
+			bev->base = NULL;
+		}
 	}
 }
 /* }}} */
 
+/* {{{ proto bool EventBufferEvent::close(void); */
+PHP_METHOD(EventBufferEvent, close)
+{
+	zval               *zbevent = getThis();
+	php_event_bevent_t *bev;
+	evutil_socket_t     fd;
+
+	PHP_EVENT_FETCH_BEVENT(bev, zbevent);
+
+	if (bev->bevent) {
+		fd = bufferevent_getfd(bev->bevent);
+		if (fd != -1) {
+			if (evutil_closesocket(fd) != -1) {
+				RETURN_TRUE;
+			}
+		}
+	}
+
+	RETURN_FALSE;
+}
+/* }}} */
+
 /* {{{ proto array EventBufferEvent::createPair(EventBase base[, int options = 0]);
  *
  * options is one of EVENT_BEV_OPT_* constants, or 0.
@@ -1058,6 +1105,9 @@ PHP_METHOD(EventBufferEvent, sslFilter)
 
 	bev->self = return_value;
 	Z_ADDREF_P(return_value);
+
+	bev->base = zbase;
+	Z_ADDREF_P(zbase);
 }
 /* }}} */
 
@@ -1134,6 +1184,9 @@ PHP_METHOD(EventBufferEvent, sslSocket)
 
 	bev->self = return_value;
 	Z_ADDREF_P(return_value);
+
+	bev->base = zbase;
+	Z_ADDREF_P(zbase);
 }
 /* }}} */
 
@@ -1189,6 +1242,116 @@ PHP_METHOD(EventBufferEvent, sslRenegotiate)
 	bufferevent_ssl_renegotiate(bev->bevent);
 }
 /* }}} */
+
+/* {{{ proto string EventBufferEvent::sslGetCipherInfo(void);
+ *
+ * Returns the current Cipher of the connexion as
+ * SSL_get_current_cipher/SSL_CIPHER_description do. Otherwise FALSE. */
+PHP_METHOD(EventBufferEvent, sslGetCipherInfo)
+{
+	zval               *zbevent = getThis();
+	php_event_bevent_t *bev;
+	struct ssl_st      *ssl;
+	char               *desc;
+	const SSL_CIPHER   *cipher;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	PHP_EVENT_FETCH_BEVENT(bev, zbevent);
+	_ret_if_invalid_bevent_ptr(bev);
+
+	ssl = bufferevent_openssl_get_ssl(bev->bevent);
+	if (ssl) {
+		cipher = SSL_get_current_cipher(ssl);
+		if (cipher) {
+			desc = SSL_CIPHER_description(cipher, NULL, 128);
+			RETVAL_STRING(desc, 1);
+			OPENSSL_free(desc);
+			return;
+		}
+	}
+
+	RETVAL_FALSE;
+}
+/* }}} */
+
+/* {{{ proto string EventBufferEvent::sslGetCipherName(void);
+ *
+ * Returns the current Cipher Name of the connection as SSL_get_cipher_name does.
+ * returns FALSE, if there is no more error to return. */
+PHP_METHOD(EventBufferEvent, sslGetCipherName)
+{
+	zval               *zbevent = getThis();
+	php_event_bevent_t *bev;
+	struct ssl_st      *ssl;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	PHP_EVENT_FETCH_BEVENT(bev, zbevent);
+	_ret_if_invalid_bevent_ptr(bev);
+
+	ssl = bufferevent_openssl_get_ssl(bev->bevent);
+	if (ssl) {
+		RETURN_STRING(SSL_get_cipher_name(ssl), 1);
+	}
+	RETVAL_FALSE;
+}
+/* }}} */
+
+/* {{{ proto string EventBufferEvent::sslGetCipherVersion(void);
+ *
+ * Returns the current cipher version of the connection as SSL_get_cipher_version does.
+ * returns FALSE, if there is no more error to return. */
+PHP_METHOD(EventBufferEvent, sslGetCipherVersion)
+{
+	zval               *zbevent  = getThis();
+	php_event_bevent_t *bev;
+	struct ssl_st      *ssl;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	PHP_EVENT_FETCH_BEVENT(bev, zbevent);
+	_ret_if_invalid_bevent_ptr(bev);
+
+	ssl = bufferevent_openssl_get_ssl(bev->bevent);
+	if (ssl) {
+		RETURN_STRING(SSL_get_cipher_version(ssl), 1);
+	}
+	RETVAL_FALSE;
+}
+/* }}} */
+
+/* {{{ proto string EventBufferEvent::sslGetProtocol(void);
+ *
+ * Returns the current Protocol of the connection as SSL_get_version does,
+ * otherwise FALSE. */
+PHP_METHOD(EventBufferEvent, sslGetProtocol)
+{
+	zval               *zbevent = getThis();
+	php_event_bevent_t *bev;
+	struct ssl_st      *ssl;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	PHP_EVENT_FETCH_BEVENT(bev, zbevent);
+	_ret_if_invalid_bevent_ptr(bev);
+
+	ssl = bufferevent_openssl_get_ssl(bev->bevent);
+	if (ssl) {
+		RETURN_STRING(SSL_get_version(ssl), 1);
+	}
+	RETVAL_FALSE;
+}
+/* }}} */
+
 #endif /* HAVE_EVENT_OPENSSL_LIB }}} */
 
 /*
